      subroutine conccalc(itime,weight)
C                           i     i
********************************************************************************
*                                                                              *
*     Calculation of the concentrations on a regular grid using volume sampling*
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     24 May 1996                                                              *
*                                                                              *
*     April 2000: Update to calculate age spectra                              *
*                 Bug fix to avoid negative conc. at the domain boundaries,    *
*                 as suggested by Petra Seibert                                *
*                                                                              *
*     2 July 2002: re-order if-statements in order to optimize CPU time        *
*                                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* nspeciesdim     = nspec for forward runs, 1 for backward runs                *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer itime,itage,i,ix,jy,ixp,jyp,kz,k,n,nage,nspeciesdim
      real weight,hx,hy,hz,h,xd,yd,zd,factor,xkern,r2,c(maxspec),ddx,ddy
      real hxmax,hymax,hzmax,xl,yl,wx,wy,w
      integer nspecpointer(maxspec)
      parameter(factor=.596831,hxmax=6.0,hymax=4.0,hzmax=150.)


C For forward simulations, make a loop over the number of species;
C for backward simulations, there is only one species, but the output
C grid is dimensioned with numpoint (number of "backward release"
C (i.e., receptor) points),and each particle makes it's contribution only
C once to this output field
****************************************************************************

      if (ldirect.eq.1) then
        nspeciesdim=nspec
        do 27 k=1,nspec
27        nspecpointer(k)=k
      else
        nspeciesdim=1
      endif


C 1. Evaluate grid concentrations using a uniform kernel of bandwidths dx, dy
*****************************************************************************

      do 20 i=1,numpart
        if (itra1(i).ne.itime) goto 20

C Determine age class of the particle
        itage=abs(itra1(i)-itramem(i))
        do 32 nage=1,nageclass
          if (itage.lt.lage(nage)) goto 33
32        continue
33      continue


C For backward simulations, look from which release point the particle comes from
*********************************************************************************

        if (ldirect.ne.1) nspecpointer(1)=npoint(i)
        
        do 30 kz=1,numzgrid                ! determine height of cell
          if (outheight(kz).gt.ztra1(i)) goto 21
30        continue
21      continue
        if (kz.le.numzgrid) then           ! inside output domain
          xl=(xtra1(i)*dx+xoutshift)/dxout
          yl=(ytra1(i)*dy+youtshift)/dyout
          ix=int(xl)
          if (xl.lt.0.) ix=ix-1
          jy=int(yl)
          if (yl.lt.0.) jy=jy-1


C For particles aged less than 3 hours, attribute particle mass to grid cell
C it resides in, in order to avoid smoothing effect of the kernel.
C For older particles, use the uniform kernel.
****************************************************************************

          if (itage.lt.10800) then             ! direct attribution to grid cell
            if ((ix.ge.0).and.(jy.ge.0).and.(ix.le.numxgrid-1).and.
     +      (jy.le.numygrid-1)) then
              do 42 k=1,nspeciesdim
42              gridunc(ix,jy,kz,nspecpointer(k),nclass(i),nage)=
     +          gridunc(ix,jy,kz,nspecpointer(k),nclass(i),nage)+
     +          xmass1(i,k)*weight
            endif

          else                                 ! attribution via uniform kernel

            ddx=xl-float(ix)                   ! distance to left cell border
            ddy=yl-float(jy)                   ! distance to lower cell border
            if (ddx.gt.0.5) then
              ixp=ix+1
              wx=1.5-ddx
            else
              ixp=ix-1
              wx=0.5+ddx
            endif

            if (ddy.gt.0.5) then
              jyp=jy+1
              wy=1.5-ddy
            else
              jyp=jy-1
              wy=0.5+ddy
            endif


C Determine mass fractions for four grid points
***********************************************

            if ((ix.ge.0).and.(ix.le.numxgrid-1)) then
              if ((jy.ge.0).and.(jy.le.numygrid-1)) then
                w=wx*wy
                do 22 k=1,nspeciesdim
22                gridunc(ix,jy,kz,nspecpointer(k),nclass(i),nage)=
     +            gridunc(ix,jy,kz,nspecpointer(k),nclass(i),nage)+
     +            xmass1(i,k)*weight*w
              endif

              if ((jyp.ge.0).and.(jyp.le.numygrid-1)) then
                w=wx*(1.-wy)
                do 25 k=1,nspeciesdim
25                gridunc(ix,jyp,kz,nspecpointer(k),nclass(i),nage)=
     +            gridunc(ix,jyp,kz,nspecpointer(k),nclass(i),nage)+
     +            xmass1(i,k)*weight*w
              endif
            endif


            if ((ixp.ge.0).and.(ixp.le.numxgrid-1)) then
              if ((jyp.ge.0).and.(jyp.le.numygrid-1)) then
                w=(1.-wx)*(1.-wy)
                do 23 k=1,nspeciesdim
23                gridunc(ixp,jyp,kz,nspecpointer(k),nclass(i),nage)=
     +            gridunc(ixp,jyp,kz,nspecpointer(k),nclass(i),nage)+
     +            xmass1(i,k)*weight*w
              endif

              if ((jy.ge.0).and.(jy.le.numygrid-1)) then
                w=(1.-wx)*wy
                do 24 k=1,nspeciesdim
24                gridunc(ixp,jy,kz,nspecpointer(k),nclass(i),nage)=
     +            gridunc(ixp,jy,kz,nspecpointer(k),nclass(i),nage)+
     +            xmass1(i,k)*weight*w
              endif

            endif

          endif
        endif
20      continue



C 2. Evaluate concentrations at receptor points, using the kernel method
************************************************************************

      do 50 n=1,numreceptor


C Reset concentrations
**********************

        do 35 k=1,nspec
35        c(k)=0.


C Estimate concentration at receptor
************************************

        do 40 i=1,numpart

          if (itra1(i).ne.itime) goto 40
          itage=abs(itra1(i)-itramem(i))

          hz=min(50.+0.3*sqrt(float(itage)),hzmax)
          zd=ztra1(i)/hz
          if (zd.gt.1.) goto 40          ! save computing time, leave loop

          hx=min((0.29+2.222e-3*sqrt(float(itage)))*dx+
     +    float(itage)*1.2e-5,hxmax)                     ! 80 km/day
          xd=(xtra1(i)-xreceptor(n))/hx
          if (xd*xd.gt.1.) goto 40       ! save computing time, leave loop

          hy=min((0.18+1.389e-3*sqrt(float(itage)))*dy+
     +    float(itage)*7.5e-6,hymax)                     ! 80 km/day
          yd=(ytra1(i)-yreceptor(n))/hy
          if (yd*yd.gt.1.) goto 40       ! save computing time, leave loop
          h=hx*hy*hz

          r2=xd*xd+yd*yd+zd*zd
          if (r2.lt.1.) then
            xkern=factor*(1.-r2)
            do 45 k=1,nspec
45            c(k)=c(k)+xmass1(i,k)*xkern/h
          endif
40        continue

        do 50 k=1,nspec
50        creceptor(n,k)=creceptor(n,k)+2.*weight*c(k)/receptorarea(n)

      end
